##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Interfaces that give the size of an object.
"""

from zope.interface import Interface


class ISized(Interface):
    """
    An object that is sized in some unit.

    Basic units:

    - 'byte'

    - 'item'  for example, number of subobjects for a folder

    - None    for unsized things

    - 'line'  for source-code like things

    """

    def sizeForSorting():
        """Returns a tuple (basic_unit, amount)

        Used for sorting among different kinds of sized objects.
        'amount' need only be sortable among things that share the
        same basic unit."""

    def sizeForDisplay():
        """Returns a string giving the size. The output string may be a
        zope.i18nmessageid.message.Message with an embedded mapping, so
        it should be translated with zope.i18n.translate()
        """



##############################################################################
#
# Copyright (c) 2001, 2002, 2011 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test ISized Adapter
"""

import unittest

import zope.component
import zope.configuration.xmlconfig

import zope.size
from zope.size.interfaces import ISized


class ZCMLTest(unittest.TestCase):

    def test_configure_zcml_should_be_loadable(self):
        try:
            zope.configuration.xmlconfig.XMLConfig(
                'configure.zcml', zope.size)()
        except Exception as e:  # pragma: no cover
            self.fail(e)

    def test_configure_should_register_n_components(self):
        gsm = zope.component.getGlobalSiteManager()
        u_count = len(list(gsm.registeredUtilities()))
        a_count = len(list(gsm.registeredAdapters()))
        s_count = len(list(gsm.registeredSubscriptionAdapters()))
        h_count = len(list(gsm.registeredHandlers()))
        zope.configuration.xmlconfig.XMLConfig(
            'configure.zcml', zope.size)()
        self.assertEqual(u_count + 8, len(list(gsm.registeredUtilities())))
        self.assertEqual(a_count + 1, len(list(gsm.registeredAdapters())))
        self.assertEqual(
            s_count, len(list(gsm.registeredSubscriptionAdapters())))
        self.assertEqual(h_count, len(list(gsm.registeredHandlers())))


class DummyObject:

    def __init__(self, size):
        self._size = size

    def getSize(self):
        return self._size


class Test(unittest.TestCase):

    def testImplementsISized(self):
        from zope.size import DefaultSized
        sized = DefaultSized(object())
        self.assertTrue(ISized.providedBy(sized))

    def testSizeWithBytes(self):
        from zope.size import DefaultSized
        obj = DummyObject(1023)
        sized = DefaultSized(obj)
        self.assertEqual(sized.sizeForSorting(), ('byte', 1023))
        self.assertEqual(sized.sizeForDisplay(), '1 KB')

    def testSizeWithNone(self):
        from zope.size import DefaultSized
        obj = DummyObject(None)
        sized = DefaultSized(obj)
        self.assertEqual(sized.sizeForSorting(), (None, None))
        self.assertEqual(sized.sizeForDisplay(), 'not-available')

    def testSizeNotAvailable(self):
        from zope.size import DefaultSized
        sized = DefaultSized(object())
        self.assertEqual(sized.sizeForSorting(), (None, None))
        self.assertEqual(sized.sizeForDisplay(), 'not-available')

    def testVariousSizes(self):
        from zope.size import DefaultSized

        sized = DefaultSized(DummyObject(0))
        self.assertEqual(sized.sizeForSorting(), ('byte', 0))
        self.assertEqual(sized.sizeForDisplay(), '0 KB')

        sized = DefaultSized(DummyObject(1))
        self.assertEqual(sized.sizeForSorting(), ('byte', 1))
        self.assertEqual(sized.sizeForDisplay(), '1 KB')

        sized = DefaultSized(DummyObject(2048))
        self.assertEqual(sized.sizeForSorting(), ('byte', 2048))
        self.assertEqual(sized.sizeForDisplay(), '${size} KB')
        self.assertEqual(sized.sizeForDisplay().mapping, {'size': '2'})

        sized = DefaultSized(DummyObject(2000000))
        self.assertEqual(sized.sizeForSorting(), ('byte', 2000000))
        self.assertEqual(sized.sizeForDisplay(), '${size} MB')
        self.assertEqual(sized.sizeForDisplay().mapping, {'size': '1.91'})

    def test_byteDisplay(self):
        from zope.size import byteDisplay
        self.assertEqual(byteDisplay(0), '0 KB')
        self.assertEqual(byteDisplay(1), '1 KB')
        self.assertEqual(byteDisplay(2048), '${size} KB')
        self.assertEqual(byteDisplay(2048).mapping, {'size': '2'})
        self.assertEqual(byteDisplay(2000000), '${size} MB')
        self.assertEqual(byteDisplay(2000000).mapping, {'size': '1.91'})



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Adapters that give the size of an object.
"""
from zope.i18nmessageid import MessageFactory
from zope.interface import implementer

from zope.size.interfaces import ISized


_ = MessageFactory('zope')


@implementer(ISized)
class DefaultSized:
    """
    A default :class:`zope.size.interfaces.ISized` adapter
    producing bytes for any object that has a ``getSize`` method and
    (None, None) for all other objects.
    """

    def __init__(self, obj):
        try:
            size = int(obj.getSize())
        except (AttributeError, ValueError, TypeError):
            self._sortingSize = None, None
        else:
            self._sortingSize = 'byte', size

    def sizeForSorting(self):
        """See ISized"""
        return self._sortingSize

    def sizeForDisplay(self):
        """See ISized"""
        units, size = self._sortingSize
        if units == 'byte':
            return byteDisplay(size)
        return _('not-available', 'n/a')


def byteDisplay(size):
    """
    Returns a size with the correct unit (KB, MB), given the size in bytes.
    The output should be given to zope.i18n.translate()
    """
    if size == 0:
        return _('0 KB')
    if size <= 1024:
        return _('1 KB')
    if size > 1048576:
        return _('${size} MB', mapping={'size': '%0.02f' % (size / 1048576.0)})
    return _('${size} KB', mapping={'size': '%d' % (size / 1024.0)})
